@@ -10,6 +10,7 @@
 #include <aws/common/ref_count.h>
 #include <aws/io/retry_strategy.h>
 #include <aws/s3/s3.h>
+#include <aws/s3/s3_buffer_pool.h>
 
 AWS_PUSH_SANE_WARNING_LEVEL
 
@@ -77,11 +78,14 @@ enum aws_s3_meta_request_type {
      * a CopyObject request to S3 if the object size is not large enough for
      * a multipart upload.
      * Note: copy support is still in development and has following limitations:
-     * - host header must use virtual host addressing style (path style is not
+     * 1. host header must use virtual host addressing style (path style is not
      *   supported) and both source and dest buckets must have dns compliant name
-     * - only {bucket}/{key} format is supported for source and passing arn as
+     * 2. only {bucket}/{key} format is supported for source and passing arn as
      *   source will not work
-     * - source bucket is assumed to be in the same region as dest
+     * 3. source bucket is assumed to be in the same region as dest
+     * 4. source bucket and dest bucket must both be either directory buckets or regular buckets.
+     *
+     * Provide the `meta_request_options.copy_source_uri` to bypass limitation 1 & 2.
      */
     AWS_S3_META_REQUEST_TYPE_COPY_OBJECT,
 
@@ -185,6 +189,30 @@ typedef void(aws_s3_meta_request_finish_fn)(
     const struct aws_s3_meta_request_result *meta_request_result,
     void *user_data);
 
+struct aws_s3_meta_request_receive_body_extra_info {
+    /* The byte index of the object that this refers to. For example, for an HTTP message that has a range header, the
+       first chunk received will have a range_start that matches the range header's range-start.*/
+    uint64_t range_start;
+
+    /* Associated buffer
+     */
+    struct aws_s3_buffer_ticket *ticket;
+};
+
+typedef int(aws_s3_meta_request_receive_body_callback_ex_fn)(
+
+    /* The meta request that the callback is being issued for. */
+    struct aws_s3_meta_request *meta_request,
+
+    /* The body data for this chunk of the object. */
+    const struct aws_byte_cursor *body,
+
+    /* Extra information associated with the delivered body */
+    const struct aws_s3_meta_request_receive_body_extra_info info,
+
+    /* User data specified by aws_s3_meta_request_options.*/
+    void *user_data);
+
 /**
  * Information sent in the meta_request progress callback.
  */
@@ -223,6 +251,20 @@ typedef void(aws_s3_meta_request_shutdown_fn)(void *user_data);
 
 typedef void(aws_s3_client_shutdown_complete_callback_fn)(void *user_data);
 
+/**
+ * Optional callback, for you to provide the full object checksum after the object was read.
+ * Client will NOT check the checksum provided before sending it to the server.
+ *
+ * @param meta_request  pointer to the aws_s3_meta_request of the upload.
+ * @param user_data     pointer to the user_data set.
+ *
+ * @return A new string with the full object checksum, as it is sent in a PutObject request (base64-encoded):
+ *         https://docs.aws.amazon.com/AmazonS3/latest/API/API_PutObject.html#API_PutObject_RequestSyntax
+ *         If an error occurs, call aws_raise_error(E) with a proper error code and return NULL.
+ */
+typedef struct aws_string *(aws_s3_meta_request_full_object_checksum_fn)(struct aws_s3_meta_request *meta_request,
+                                                                         void *user_data);
+
 enum aws_s3_meta_request_tls_mode {
     AWS_MR_TLS_ENABLED,
     AWS_MR_TLS_DISABLED,
@@ -240,7 +282,8 @@ enum aws_s3_checksum_algorithm {
     AWS_SCA_CRC32,
     AWS_SCA_SHA1,
     AWS_SCA_SHA256,
-    AWS_SCA_END = AWS_SCA_SHA256,
+    AWS_SCA_CRC64NVME,
+    AWS_SCA_END = AWS_SCA_CRC64NVME,
 };
 
 enum aws_s3_checksum_location {
@@ -249,7 +292,7 @@ enum aws_s3_checksum_location {
     AWS_SCL_TRAILER,
 };
 
-enum aws_s3_recv_file_option {
+enum aws_s3_recv_file_options {
     /**
      * Create a new file if it doesn't exist, otherwise replace the existing file.
      */
@@ -271,6 +314,43 @@ enum aws_s3_recv_file_option {
      */
     AWS_S3_RECV_FILE_WRITE_TO_POSITION,
 };
+
+/**
+ * WARNING: experimental/unstable:
+ * Controls how client performance file I/O operations. Only applies to the file based workload.
+ **/
+struct aws_s3_file_io_options {
+    /**
+     * Skip buffering the part in memory before sending the request.
+     *
+     * Default to false on small objects, and true when the object size exceed a certain threshold
+     *`g_streaming_object_size_threshold`.
+     **/
+    bool should_stream;
+
+    /**
+     * The estimated disk throughput. Only be applied when `streaming_upload` is true.
+     * in gigabits per second (Gbps).
+     *
+     * Notes: There are possibilities that cannot reach the all available disk throughput:
+     * 1. Disk is busy with other applications
+     * 2. OS Cache may cap the throughput, use `direct_io` to get around this.
+     *
+     * Default to throughput_target_gbps.
+     **/
+    double disk_throughput_gbps;
+
+    /**
+     * Enable direct IO to bypass the OS cache. Helpful when the disk I/O outperforms the kernel cache.
+     * Notes:
+     * - Only supported on linux for now.
+     * - Only supports upload for now.
+     * - Check NOTES for O_DIRECT for additional info https://man7.org/linux/man-pages/man2/openat.2.html
+     * In summary, O_DIRECT is a potentially powerful tool that should be used with caution.
+     */
+    bool direct_io;
+};
+
 /**
  * Info about a single part, for you to review before the upload completes.
  */
@@ -395,6 +475,20 @@ struct aws_s3_client_config {
      */
     const struct aws_tls_connection_options *tls_connection_options;
 
+    /**
+     * Optional.
+     * If set, this controls how the client interact with file I/O.
+     * Read `aws_s3_file_io_options` for details.
+     *  Notes: Only applies to meta requests with `send_filepath` set.
+     *  TODO: adapt it to `recv_filepath`.
+     *
+     * eg:
+     * - When the file is too large to fit in the buffer, set `should_stream` to avoid buffering the whole parts in
+     *  memory.
+     * - When the disk I/O is faster than OS cache, set `direct_io` to bypass the OS cache.
+     */
+    struct aws_s3_file_io_options *fio_opts;
+
     /**
      * Required.
      * Configure the signing for the requests made from the client.
@@ -416,7 +510,9 @@ struct aws_s3_client_config {
      * Optional.
      * Size of parts the object will be downloaded or uploaded in, in bytes.
      * This only affects AWS_S3_META_REQUEST_TYPE_GET_OBJECT and AWS_S3_META_REQUEST_TYPE_PUT_OBJECT.
-     * If not set, this defaults to 8 MiB.
+     *
+     * If not set, a dynamic default part size will be used based on the throughput target, memory_limit_in_bytes.
+     *
      * The client will adjust the part size for AWS_S3_META_REQUEST_TYPE_PUT_OBJECT if needed for service limits (max
      * number of parts per upload is 10,000, minimum upload part size is 5 MiB).
      *
@@ -424,9 +520,7 @@ struct aws_s3_client_config {
      */
     uint64_t part_size;
 
-    /* If the part size needs to be adjusted for service limits, this is the maximum size it will be adjusted to. On 32
-     * bit machine, it will be forced to SIZE_MAX, which is around 4GiB. The server limit is 5GiB, but object size limit
-     * is 5TiB for now. We should be good enough for all the cases. */
+    /* If the part size needs to be adjusted for service limits, this is the maximum size it will be adjusted to. */
     uint64_t max_part_size;
 
     /**
@@ -534,6 +628,10 @@ struct aws_s3_client_config {
      * If set, client will invoke the factory to get the provider to use, when needed.
      *
      * If not set, client will create a default S3 Express provider under the hood.
+     *
+     * NOTE: THE FOLLOWING BEHAVIOR IS EXPERIMENTAL AND UNSTABLE
+     * Default S3 Express provider will pass the headers allowed in `g_s3_create_session_allowed_headers` to the
+     * CreateSession call.
      */
     aws_s3express_provider_factory_fn *s3express_provider_override_factory;
     void *factory_user_data;
@@ -551,29 +649,60 @@ struct aws_s3_client_config {
      */
     const struct aws_byte_cursor *network_interface_names_array;
     size_t num_network_interface_names;
+
+    /**
+     * Experimental (touch at your own risk).
+     * (Optional)
+     * Factory function to create buffer pool.
+     * if not set creates default implementation of the buffer pool.
+     */
+    aws_s3_buffer_pool_factory_fn *buffer_pool_factory_fn;
+
+    /* User data thats passed into pool factory. */
+    void *buffer_pool_user_data;
 };
 
 struct aws_s3_checksum_config {
 
+    /****************************** PUT Object specific *******************************/
     /**
      * The location of client added checksum header.
      *
-     * If AWS_SCL_NONE. No request payload checksum will be add and calculated.
+     * If AWS_SCL_NONE. No request payload checksum will be added.
      *
-     * If AWS_SCL_HEADER, the checksum will be calculated by client and added related header to the request sent.
+     * If AWS_SCL_HEADER, the client will calculate the checksum and add it to the headers.
      *
-     * If AWS_SCL_TRAILER, the payload will be aws_chunked encoded, The checksum will be calculate while reading the
-     * payload by client. Related header will be added to the trailer part of the encoded payload. Note the payload of
-     * the original request cannot be aws-chunked encoded already. Otherwise, error will be raised.
+     * If AWS_SCL_TRAILER, the payload will be aws_chunked encoded, The client will calculate the checksum and add it to
+     * the trailer. Note the payload of the original request cannot be aws-chunked encoded already, this will cause an
+     * error.
      */
     enum aws_s3_checksum_location location;
 
     /**
      * The checksum algorithm used.
-     * Must be set if location is not AWS_SCL_NONE. Must be AWS_SCA_NONE if location is AWS_SCL_NONE.
+     * Must be set if location is not AWS_SCL_NONE.
      */
     enum aws_s3_checksum_algorithm checksum_algorithm;
 
+    /**
+     * Optional.
+     * Provide the full object checksum. This callback is invoked once, after the entire body has been read.
+     * sent.
+     *
+     * NOTE:
+     *  - Do not set this callback if the HTTP message already has a checksum header (e.g. x-amz-checksum-crc32). Doing
+     *      so will raise AWS_ERROR_INVALID_ARGUMENT.
+     *  - checksum_algorithm must be set to the algorithm you will use.
+     *
+     * WARNING: This feature is experimental/unstable.
+     * At this time, full object checksum callback is only available for multipart upload
+     * (when Content-Length is above the `multipart_upload_threshold`,
+     * or Content-Length not specified). Otherwise, it will be ignored.
+     */
+    aws_s3_meta_request_full_object_checksum_fn *full_object_checksum_callback;
+    void *user_data;
+
+    /****************************** GET Object specific *******************************/
     /**
      * Enable checksum mode header will be attached to GET requests, this will tell s3 to send back checksums headers if
      * they exist. Calculate the corresponding checksum on the response bodies. The meta request will finish with a did
@@ -591,7 +720,7 @@ struct aws_s3_checksum_config {
      *
      * The list of algorithms for user to pick up when validate the checksum. Client will pick up the algorithm from the
      * list with the priority based on performance, and the algorithm sent by server. The priority based on performance
-     * is [CRC32C, CRC32, SHA1, SHA256].
+     * is [CRC64NVME, CRC32C, CRC32, SHA1, SHA256].
      *
      * If the response checksum was validated by client, the result will indicate which algorithm was picked.
      */
@@ -659,7 +788,7 @@ struct aws_s3_meta_request_options {
      * If set, the received data will be written into this file.
      * the `body_callback` will NOT be invoked.
      * This gives a better performance when receiving data to write to a file.
-     * See `aws_s3_recv_file_option` for the configuration on the receive file.
+     * See `aws_s3_recv_file_options` for the configuration on the receive file.
      */
     struct aws_byte_cursor recv_filepath;
 
@@ -667,9 +796,9 @@ struct aws_s3_meta_request_options {
      * Optional.
      * Default to AWS_S3_RECV_FILE_CREATE_OR_REPLACE.
      * This only works with recv_filepath set.
-     * See `aws_s3_recv_file_option`.
+     * See `aws_s3_recv_file_options`.
      */
-    enum aws_s3_recv_file_option recv_file_option;
+    enum aws_s3_recv_file_options recv_file_option;
     /**
      * Optional.
      * The specified position to start writing at for the recv file when `recv_file_option` is set to
@@ -690,6 +819,24 @@ struct aws_s3_meta_request_options {
      */
     struct aws_byte_cursor send_filepath;
 
+    /**
+     * Optional.
+     * Overrides the client config if set.
+     * If set, this controls how the meta request interact with file I/O.
+     * Read `aws_s3_file_io_options` for details.
+     *  Notes: Only applies when `send_filepath` is set.
+     *  TODO: adapt it to `recv_filepath`.
+     *
+     * Note: if both client and meta request don't set this, for objects larger than 2TiB, this will be set to a default
+     * options with `should_stream` to be True and others follow the default to avoid memory issues.
+     *
+     * eg:
+     * - When the file is too large to fit in the buffer, set `should_stream` to avoid buffering the whole parts in
+     *  memory.
+     * - When the disk I/O is faster than OS cache, set `direct_io` to bypass the OS cache.
+     */
+    struct aws_s3_file_io_options *fio_opts;
+
     /**
      * Optional - EXPERIMENTAL/UNSTABLE
      * If set, the request body comes from this async stream.
@@ -720,6 +867,9 @@ struct aws_s3_meta_request_options {
     /**
      * Optional.
      * if set, the flexible checksum will be performed by client based on the config.
+     *
+     * Notes: checksum can also be added through the http message provided.
+     *      The checksum in http header will override corresponding the checksum config.
      */
     const struct aws_s3_checksum_config *checksum_config;
 
@@ -727,13 +877,22 @@ struct aws_s3_meta_request_options {
      * Optional.
      * Size of parts the object will be downloaded or uploaded in, in bytes.
      * This only affects AWS_S3_META_REQUEST_TYPE_GET_OBJECT and AWS_S3_META_REQUEST_TYPE_PUT_OBJECT.
-     * If not set, the value from `aws_s3_client_config.part_size` is used, which defaults to 8MiB.
+     *
+     * If not set, the value from `aws_s3_client_config.part_size` is used, which defaults to a dynamic value based on
+     * the throughput target, memory_limit_in_bytes and the requested object size.
      *
      * The client will adjust the part size for AWS_S3_META_REQUEST_TYPE_PUT_OBJECT if needed for service limits (max
      * number of parts per upload is 10,000, minimum upload part size is 5 MiB).
      */
     uint64_t part_size;
 
+    /**
+     * Optional - EXPERIMENTAL/UNSTABLE
+     * Set this to prefer for the dynamic default part_size over the part size set for both client and meta request for
+     * the best performance under the memory constrain, especially for getting large objects.
+     */
+    bool force_dynamic_part_size;
+
     /**
      * Optional.
      * The size threshold in bytes for when to use multipart uploads.
@@ -766,6 +925,15 @@ struct aws_s3_meta_request_options {
      */
     aws_s3_meta_request_receive_body_callback_fn *body_callback;
 
+    /**
+     * Invoked to provide the response body as it is received.
+     * Provides extra information as compared to regular callback.
+     * Note: setting both body_callback and body_callback_ex is not
+     * considered valid and meta request creation will fail.
+     * See `aws_s3_meta_request_receive_body_callback_ex_fn`.
+     */
+    aws_s3_meta_request_receive_body_callback_ex_fn *body_callback_ex;
+
     /**
      * Invoked when the entire meta request execution is complete.
      * See `aws_s3_meta_request_finish_fn`.
@@ -794,7 +962,7 @@ struct aws_s3_meta_request_options {
      * Optional.
      * Callback for reviewing an upload before it completes.
      * WARNING: experimental/unstable
-     * See `aws_s3_upload_review_fn`
+     * See `aws_s3_meta_request_upload_review_fn`
      */
     aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
@@ -830,6 +998,17 @@ struct aws_s3_meta_request_options {
      * This is just used as an estimate, so it's okay to provide an approximate value if the exact size is unknown.
      */
     const uint64_t *object_size_hint;
+
+    /*
+     * (Optional)
+     * If performing a copy operation, provide the source URI here to bypass limitations 1 and 2 of the copy operation.
+     * This will be ignored for other operations.
+     */
+    struct aws_byte_cursor copy_source_uri;
+
+    /* When set, this will cap the number of active connections for the meta request. When 0, the client will determine
+     * it based on client side settings. (Recommended) */
+    uint32_t max_active_connections_override;
 };
 
 /* Result details of a meta request.
@@ -1226,6 +1405,17 @@ int aws_s3_request_metrics_get_request_id(
     const struct aws_s3_request_metrics *metrics,
     const struct aws_string **out_request_id);
 
+/**
+ * Get the extended request ID from aws_s3_request_metrics.
+ * If unavailable, AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised.
+ * If available, out_extended_request_id will be set to a string. Be warned this string's lifetime is tied to the
+ * metrics object.
+ **/
+AWS_S3_API
+int aws_s3_request_metrics_get_extended_request_id(
+    const struct aws_s3_request_metrics *metrics,
+    const struct aws_string **out_extended_request_id);
+
 /* Get the start time from aws_s3_request_metrics, which is when S3 client prepare the request to be sent. Always
  * available. Timestamp are from `aws_high_res_clock_get_ticks`  */
 AWS_S3_API
@@ -1279,6 +1469,72 @@ int aws_s3_request_metrics_get_receive_end_timestamp_ns(
     const struct aws_s3_request_metrics *metrics,
     uint64_t *out_receive_end_time);
 
+/* Get the time stamp when the signing for the request started. Timestamps are from
+ * `aws_high_res_clock_get_ticks` AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_sign_start_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_signing_start_time);
+
+/* Get the time stamp when the signing for the request ended. Timestamps are from
+ * `aws_high_res_clock_get_ticks` AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_sign_end_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_signing_end_time);
+
+/* The time duration for the request singing (sign_end_timestamp_ns -
+ * sign_start_timestamp_ns).
+ * AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_signing_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_signing_duration);
+
+/* Get the time stamp when the memory acquisition for the request started. Timestamps are from
+ * `aws_high_res_clock_get_ticks` AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_mem_acquire_start_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_mem_acquire_start_time);
+
+/* Get the time stamp when the memory acquisition for the request ended. Timestamps are from
+ * `aws_high_res_clock_get_ticks` AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_mem_acquire_end_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_mem_acquire_end_time);
+
+/* The time duration for the request memory acquisition (mem_acquire_end_timestamp_ns -
+ * mem_acquire_start_timestamp_ns).
+ * AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_mem_acquire_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_mem_acquire_duration);
+
+/* Get the time stamp when the delivery of the request response started. Timestamps are from
+ * `aws_high_res_clock_get_ticks` AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_delivery_start_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_delivery_start_time);
+
+/* Get the time stamp when the delivery of the request response ended. Timestamps are from
+ * `aws_high_res_clock_get_ticks` AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_delivery_end_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_delivery_end_time);
+
+/* The time duration for the request delivery(delivery_end_timestamp_ns -
+ * delivery_start_timestamp_ns).
+ * AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_delivery_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_delivery_duration);
+
 /* The time duration for the request from start receiving to finish receiving (receive_end_timestamp_ns -
  * receive_start_timestamp_ns).
  * AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
@@ -1333,10 +1589,14 @@ int aws_s3_request_metrics_get_ip_address(
     const struct aws_s3_request_metrics *metrics,
     const struct aws_string **out_ip_address);
 
-/* Get the id of connection that request was made from. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data
- * not available */
+/* Get the ptr address of connection that request was made from. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised
+ * if data not available */
 AWS_S3_API
-int aws_s3_request_metrics_get_connection_id(const struct aws_s3_request_metrics *metrics, size_t *out_connection_id);
+int aws_s3_request_metrics_get_connection_id(const struct aws_s3_request_metrics *metrics, size_t *out_connection_ptr);
+
+/* Get the pointer to the request that attempt was made from. Always available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_request_ptr(const struct aws_s3_request_metrics *metrics, size_t *out_request_ptr);
 
 /* Get the thread ID of the thread that request was made from. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if
  * data not available */
@@ -1370,6 +1630,93 @@ void aws_s3_request_metrics_get_request_type(
 AWS_S3_API
 int aws_s3_request_metrics_get_error_code(const struct aws_s3_request_metrics *metrics);
 
+/* Get retry attempt from request metrics. */
+AWS_S3_API
+uint32_t aws_s3_request_metrics_get_retry_attempt(const struct aws_s3_request_metrics *metrics);
+
+/* Get whether the memory for the request was allocated from the pool. Cannot fail */
+AWS_S3_API
+bool aws_s3_request_metrics_get_memory_allocated_from_pool(const struct aws_s3_request_metrics *metrics);
+
+/* Get the beginning range of this part from request metrics. */
+AWS_S3_API
+void aws_s3_request_metrics_get_part_range_start(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_part_range_start);
+
+/* Get the last byte of this part from request metrics. */
+AWS_S3_API
+void aws_s3_request_metrics_get_part_range_end(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_part_range_end);
+
+/* Get the part number from request metrics. */
+AWS_S3_API
+void aws_s3_request_metrics_get_part_number(const struct aws_s3_request_metrics *metrics, uint32_t *out_part_number);
+/* Get the request start timestamp from aws_s3_request_metrics. Always available. */
+AWS_S3_API
+void aws_s3_request_metrics_get_s3_request_first_attempt_start_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_s3_request_first_attempt_start_time);
+
+/* Get the request end timestamp. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not
+ * available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_s3_request_last_attempt_end_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_s3_request_last_attempt_end_time);
+
+/* Get the request duration. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not
+ * available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_s3_request_total_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_request_duration);
+
+/* Get the connection acquire start timestamp. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not
+ * available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_conn_acquire_start_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_conn_acquire_start_time);
+
+/* Get the connection acquire end timestamp. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not
+ * available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_conn_acquire_end_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_conn_acquire_end_time);
+
+/* Get the connection acquire duration. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_conn_acquire_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_conn_acquire_duration);
+
+/* Get the retry delay start timestamp. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_retry_delay_start_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_retry_delay_start_time);
+
+/* Get the retry delay end timestamp. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_retry_delay_end_timestamp_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_retry_delay_end_time);
+
+/* Get the retry delay duration. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_retry_delay_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_retry_delay_duration);
+
+/* Get the service call duration. AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised if data not available. */
+AWS_S3_API
+int aws_s3_request_metrics_get_service_call_duration_ns(
+    const struct aws_s3_request_metrics *metrics,
+    uint64_t *out_service_call_duration);
+
 AWS_EXTERN_C_END
 AWS_POP_SANE_WARNING_LEVEL
 
